@@ -14,8 +14,6 @@
 #' @param pal Colours to use. A character vector of hex codes (or names).
 #' @param pal_na Colour to use for NA values. A character vector of a hex code (or name).
 #' @param alpha Opacity. A number between 0 and 1.
-#' @param height Height. A number 0 upwards.
-#' @param width Width. A number 0 upwards.
 #' @param ... Other arguments passed to the relevant ggplot2::geom_* function.
 #' @param titles A function to format the x, y and col titles, including in rlang lambda format. Defaults to snakecase::to_sentence_case.
 #' @param title Title string.
@@ -25,8 +23,7 @@
 #' @param x_expand Padding to the limits with the ggplot2::expansion function, or a vector of length 2 (e.g. c(0, 0)).
 #' @param x_include For a numeric or date variable, any values that the scale should include (e.g. 0).
 #' @param x_labels A function that takes the breaks as inputs (e.g. scales::label_comma()), or a vector of labels.
-#' @param x_limits A vector of length 2 to determine the limits of the axis.
-#' @param x_oob A scales::oob_* function for how to deal with out-of-bounds values.
+#' @param x_limits A vector of length 2 to determine the limits of the axis. Alternatively, zoom in using coord = coord_cartesian(xlim = ...).
 #' @param x_sec_axis A secondary axis specified by the ggplot2::sec_axis or ggplot2::dup_axis function.
 #' @param x_title Axis title string. Defaults to converting to sentence case with spaces. Use "" for no title.
 #' @param x_trans For a numeric variable, a transformation object (e.g. "log10").
@@ -34,24 +31,27 @@
 #' @param y_expand Padding to the limits with the ggplot2::expansion function, or a vector of length 2 (e.g. c(0, 0)).
 #' @param y_include For a numeric or date variable, any values that the scale should include (e.g. 0).
 #' @param y_labels A function that takes the breaks as inputs (e.g. scales::label_comma()), or a vector of labels.
-#' @param y_limits A vector of length 2 to determine the limits of the axis.
-#' @param y_oob A scales::oob_* function for how to deal with out-of-bounds values.
+#' @param y_limits A vector of length 2 to determine the limits of the axis. Alternatively, zoom in using coord = coord_cartesian(ylim = ...).
 #' @param y_sec_axis A secondary axis specified by the ggplot2::sec_axis or ggplot2::dup_axis function.
 #' @param y_title Axis title string. Defaults to converting to sentence case with spaces. Use "" for no title.
 #' @param y_trans For a numeric variable, a transformation object (e.g. "log10").
 #' @param col_breaks A function that takes the limits as input (e.g. scales::breaks_pretty()), or a vector of breaks.
+#' @param col_continuous Type of colouring for a continuous variable. Either "gradient" or "steps". Defaults to "steps".
 #' @param col_include For a numeric or date variable, any values that the scale should include (e.g. 0).
-#' @param col_intervals A function to cut or chop the numeric variable into intervals (e.g. ~ santoku::chop_mean_sd(.x, drop = FALSE)).
 #' @param col_labels A function that takes the breaks as inputs (e.g. scales::label_comma()), or a vector of labels. Note this does not affect where col_intervals is not NULL.
 #' @param col_limits A vector to determine the limits of the axis.
 #' @param col_legend_ncol The number of columns for the legend elements.
 #' @param col_legend_nrow The number of rows for the legend elements.
 #' @param col_legend_place The place for the legend. "b" for bottom, "r" for right, "t" for top, or "l" for left.
-#' @param col_title Axis title string. Defaults to converting to sentence case with spaces. Use "" for no title.
-#' @param facet_labels A function that takes the breaks as inputs (e.g. scales::label_comma()), or a named vector of labels (e.g. c(value = "label", ...)).
-#' @param facet_ncol The number of columns of facetted plots.
-#' @param facet_nrow The number of rows of facetted plots.
-#' @param facet_scales Whether facet_scales should be "fixed" across facets, "free" in both directions, or free in just one direction (i.e. "free_x" or "free_y"). Defaults to "fixed".
+#' @param col_legend_rev Reverse the elements of the legend. Defaults to FALSE.
+#' @param col_legend_place The place for the legend. "b" for bottom, "r" for right, "t" for top, or "l" for left. Defaults to "b".
+#' @param col_title Legend title string. Defaults to converting to sentence case with spaces. Use "" for no title.
+#' @param facet_labels A function that takes the breaks as inputs (e.g. scales::label_comma()), or a named vector of labels (e.g. c("value" = "label", ...)).
+#' @param facet_ncol The number of columns of facets. Only applies to a facet layout of "wrap".
+#' @param facet_nrow The number of rows of facets. Only applies to a facet layout of "wrap".
+#' @param facet_scales Whether facet scales should be "fixed" across facets, "free" in both directions, or free in just one direction (i.e. "free_x" or "free_y"). Defaults to "fixed".
+#' @param facet_space Whether facet_space should be "fixed" across facets, "free" to be proportional in both directions, or free to be proportional in just one direction (i.e. "free_x" or "free_y"). Only applies where the facet layout is a 'grid' and facet_scales are not fixed. Defaults to "fixed".
+#' @param facet_layout Whether the layout is to be "wrap" or "grid". If NULL and a single facet (or facet2) argument is provided, then defaults to "wrap". If NULL and both facet and facet2 arguments are provided, defaults to "grid".
 #' @param caption Caption title string.
 #' @param theme A ggplot2 theme.
 #' @return A ggplot object.
@@ -66,7 +66,9 @@
 #'   w = rep(diff(c(0, 4, 6, 8, 10, 14)), 2)
 #' )
 #'
-#' gg_tile(df, x = x, y = y, col = z)
+#' df %>%
+#' gg_tile(x = x, y = y, col = z,
+#'         height = 0.9, width = 0.9)
 #'
 gg_tile <- function(
     data = NULL,
@@ -83,8 +85,6 @@ gg_tile <- function(
     pal_na = "#7F7F7F",
     alpha = 0.9,
     #linewidth = 0.5,
-    height = 1,
-    width = 1,
     ...,
     titles = NULL,
     title = NULL,
@@ -95,7 +95,6 @@ gg_tile <- function(
     x_include = NULL,
     x_labels = NULL,
     x_limits = NULL,
-    x_oob = scales::oob_keep,
     x_sec_axis = ggplot2::waiver(),
     x_title = NULL,
     x_trans = "identity",
@@ -104,23 +103,25 @@ gg_tile <- function(
     y_include = NULL,
     y_labels = NULL,
     y_limits = NULL,
-    y_oob = scales::oob_keep,
     y_sec_axis = ggplot2::waiver(),
     y_title = NULL,
     y_trans = "identity",
     col_breaks = NULL,
+    col_continuous = "gradient",
     col_include = NULL,
-    col_intervals = NULL,
     col_labels = NULL,
     col_legend_place = NULL,
     col_legend_ncol = NULL,
     col_legend_nrow = NULL,
+    col_legend_rev = FALSE,
     col_limits = NULL,
     col_title = NULL,
     facet_labels = NULL,
     facet_ncol = NULL,
     facet_nrow = NULL,
     facet_scales = "fixed",
+    facet_space = "fixed",
+    facet_layout = NULL,
     caption = NULL,
     theme = NULL) {
 
@@ -134,16 +135,38 @@ gg_tile <- function(
   text <- rlang::enquo(text)
 
   #stop, warn or message
+  rlang::inform(c("i" = "For further ggblanket information, see https://davidhodge931.github.io/ggblanket/"), .frequency = "regularly", .frequency_id = "hello")
   if (rlang::is_null(data)) rlang::abort("data is required.")
-  if (rlang::is_null(titles)) rlang::inform(c("i" = "{ggblanket} converts unspecified titles using snakecase::to_sentence_case. Use titles = ~.x to leave unspecified titles as is, and/or specify individual titles manually using *_title arguments."))
-  if (!rlang::quo_is_null(col)) rlang::inform(c("i" = "{ggblanket} merges col and fill aesthetics into a single col aesthetic."))
-  if (!rlang::quo_is_null(facet)) rlang::inform(c("i" = "{ggblanket} treats faceting as an aesthetic."))
 
   ###ungroup
   data <- dplyr::ungroup(data)
 
+  #get classes
+  x_character <- is.character(rlang::eval_tidy(x, data))
+  x_factor <- is.factor(rlang::eval_tidy(x, data))
+  x_logical <- is.logical(rlang::eval_tidy(x, data))
+  x_date <- lubridate::is.Date(rlang::eval_tidy(x, data))
+  x_numeric <- is.numeric(rlang::eval_tidy(x, data))
+  x_null <- rlang::quo_is_null(x)
+
+  y_character <- is.character(rlang::eval_tidy(y, data))
+  y_factor <- is.factor(rlang::eval_tidy(y, data))
+  y_date <- lubridate::is.Date(rlang::eval_tidy(y, data))
+  y_logical <- is.logical(rlang::eval_tidy(y, data))
+  y_numeric <- is.numeric(rlang::eval_tidy(y, data))
+  y_null <- rlang::quo_is_null(y)
+
+  col_character <- is.character(rlang::eval_tidy(col, data))
+  col_factor <- is.factor(rlang::eval_tidy(col, data))
+  col_logical <- is.logical(rlang::eval_tidy(col, data))
+  col_numeric <- is.numeric(rlang::eval_tidy(col, data))
+  col_null <- rlang::quo_is_null(col)
+
+  facet_null <- rlang::quo_is_null(facet)
+  facet2_null <- rlang::quo_is_null(facet2)
+
   ###get default NULL values
-  if (rlang::quo_is_null(x)) {
+  if (x_null) {
     if (rlang::is_null(x_title)) {
       if (stat %in% c("bin", "count")) {
         if (rlang::is_null(titles)) x_title <- purrr::map_chr("count", snakecase::to_sentence_case)
@@ -168,7 +191,7 @@ gg_tile <- function(
     else x_title <- purrr::map_chr(rlang::as_name(x), titles)
   }
 
-  if (rlang::quo_is_null(y)) {
+  if (y_null) {
     if (rlang::is_null(y_title)) {
       if (stat %in% c("bin", "count")) {
         if (rlang::is_null(titles)) y_title <- purrr::map_chr("count", snakecase::to_sentence_case)
@@ -193,76 +216,34 @@ gg_tile <- function(
     else y_title <- purrr::map_chr(rlang::as_name(y), titles)
   }
 
-  xy_numeric_date <- ifelse(((
-    is.numeric(rlang::eval_tidy(x, data)) |
-      rlang::quo_is_null(x) |
-      lubridate::is.Date(rlang::eval_tidy(x, data))
-  ) &
-    (
-      is.numeric(rlang::eval_tidy(y, data)) |
-        rlang::quo_is_null(y) |
-        lubridate::is.Date(rlang::eval_tidy(y, data))
-    )),
-  TRUE,
-  FALSE
-  )
-
   if (rlang::is_null(theme)) {
-    if (xy_numeric_date) {
-      grid_v <- FALSE
-      grid_h <- TRUE
-    }
-    else {
-      grid_v <-
-        ifelse(is.numeric(rlang::eval_tidy(x, data)) |
-                 lubridate::is.Date(rlang::eval_tidy(x, data)) |
-                 rlang::quo_is_null(x),
-               TRUE,
-               FALSE)
-      grid_h <-
-        ifelse(is.numeric(rlang::eval_tidy(y, data)) |
-                 lubridate::is.Date(rlang::eval_tidy(y, data)) |
-                 rlang::quo_is_null(y),
-               TRUE,
-               FALSE)
-    }
-
-    theme <- gg_theme(grid_v = grid_v, grid_h = grid_h)
+    if ((x_character | x_factor | x_logical) & (y_numeric | y_null)) theme <- gg_theme(grid_h = TRUE, grid_v = FALSE)
+    else if ((y_character | y_factor | y_logical) & (x_numeric | x_null)) theme <- gg_theme(grid_h = FALSE, grid_v = TRUE)
+    else if ((x_character | x_factor | x_logical) & (y_character | y_factor | y_logical)) theme <- gg_theme(grid_h = FALSE, grid_v = FALSE)
+    else if ((x_numeric | x_date) & (y_date | y_numeric | y_null)) theme <- gg_theme(grid_h = TRUE, grid_v = FALSE)
+    else if ((y_numeric | y_date) & (x_null)) theme <- gg_theme(grid_h = FALSE, grid_v = TRUE)
   }
 
-  # if (rlang::is_null(width)) {
-  #   if (lubridate::is.Date(rlang::eval_tidy(x, data)) |
-  #       lubridate::is.Date(rlang::eval_tidy(y, data)) |
-  #       (rlang::quo_is_null(y) & is.numeric(rlang::eval_tidy(x, data))) |
-  #       (rlang::quo_is_null(x) & is.numeric(rlang::eval_tidy(y, data))) |
-  #       (is.numeric(rlang::eval_tidy(x, data)) &
-  #        is.numeric(rlang::eval_tidy(y, data)))) {
-  #     width <- NULL
-  #   }
-  #   else
-  #     width <- 0.75
-  # }
-
   if (rlang::is_null(coord)) coord <- ggplot2::coord_cartesian(clip = "off")
 
   ###process plot data
   ###factorise logical, reverse for horizontal, and chop intervals
-  if (!rlang::quo_is_null(x)) {
-    if (is.logical(rlang::eval_tidy(x, data))) {
+  if (!x_null) {
+    if (x_logical) {
       data <- data %>%
         dplyr::mutate(dplyr::across(!!x, ~ factor(.x, levels = c("FALSE", "TRUE"))))
     }
   }
 
-  if (!rlang::quo_is_null(y)) {
-    if (is.logical(rlang::eval_tidy(y, data))) {
+  if (!y_null) {
+    if (y_logical) {
       data <- data %>%
         dplyr::mutate(dplyr::across(!!y, ~ factor(.x, levels = c("FALSE", "TRUE"))))
     }
 
-    if (is.character(rlang::eval_tidy(y, data)) | is.factor(rlang::eval_tidy(y, data))) {
+    if (y_character | y_factor) {
 
-      if (!rlang::quo_is_null(col) &
+      if (!col_null &
           (identical(rlang::eval_tidy(y, data), rlang::eval_tidy(col, data)))) {
       }
       else {
@@ -272,28 +253,28 @@ gg_tile <- function(
     }
   }
 
-  if (!rlang::quo_is_null(col)) {
+  if (!col_null) {
 
-    if (is.logical(rlang::eval_tidy(col, data))) {
+    if (col_logical) {
       data <- data %>%
         dplyr::mutate(dplyr::across(!!col, ~ factor(.x, levels = c("FALSE", "TRUE"))))
     }
 
-    if (is.character(rlang::eval_tidy(col, data)) | is.factor(rlang::eval_tidy(col, data))) {
-      if (is.factor(rlang::eval_tidy(y, data)) | is.character(rlang::eval_tidy(y, data))) {
+    if (col_character | is.factor(rlang::eval_tidy(col, data))) {
+      if (y_factor | y_character) {
         data <- data %>%
           dplyr::mutate(dplyr::across(!!col, ~ forcats::fct_rev(.x)))
       }
     }
   }
 
-  if (!rlang::quo_is_null(facet)) {
+  if (!facet_null) {
     if (is.logical(class(rlang::eval_tidy(facet, data)))) {
       data <- data %>%
         dplyr::mutate(dplyr::across(!!facet, ~ factor(.x, levels = c("FALSE", "TRUE"))))
     }
   }
-  if (!rlang::quo_is_null(facet2)) {
+  if (!facet2_null) {
     if (is.logical(class(rlang::eval_tidy(facet2, data)))) {
       data <- data %>%
         dplyr::mutate(dplyr::across(!!facet2, ~ factor(.x, levels = c("FALSE", "TRUE"))))
@@ -301,7 +282,7 @@ gg_tile <- function(
   }
 
   ###make col scale
-  if (rlang::quo_is_null(col)) {
+  if (col_null) {
     if (rlang::is_null(pal)) pal <-  pal_viridis_mix(1)
     else pal <- pal[1]
 
@@ -327,48 +308,59 @@ gg_tile <- function(
     col_title_position <- ifelse(col_title == "", "right", "top")
 
     if (rlang::is_null(col_legend_place)) {
-      if (!rlang::quo_is_null(x) &
+      if (!x_null &
           (identical(rlang::eval_tidy(col, data), rlang::eval_tidy(x, data)))) {
         col_legend_place <- "n"
       }
-      else if (!rlang::quo_is_null(y) &
+      else if (!y_null &
                (identical(rlang::eval_tidy(col, data), rlang::eval_tidy(y, data)))) {
         col_legend_place <- "n"
       }
-      else if (!rlang::quo_is_null(facet) &
+      else if (!facet_null &
                (identical(rlang::eval_tidy(col, data), rlang::eval_tidy(facet, data)))) {
         col_legend_place <- "n"
       }
-      else if (!rlang::quo_is_null(facet2) &
+      else if (!facet2_null &
                (identical(rlang::eval_tidy(col, data), rlang::eval_tidy(facet2, data)))) {
         col_legend_place <- "n"
       }
-      else
+      else {
         col_legend_place <- "b"
+      }
     }
 
-    if (is.numeric(rlang::eval_tidy(col, data))) {
-      if (rlang::is_null(col_intervals)) { #continuous col
-        col_min <- data %>% dplyr::pull(!!col) %>% min(na.rm = TRUE)
-        col_max <- data %>% dplyr::pull(!!col) %>% max(na.rm = TRUE)
+    if (col_numeric) {
+      col_min <- data %>% dplyr::pull(!!col) %>% min(na.rm = TRUE)
+      col_max <- data %>% dplyr::pull(!!col) %>% max(na.rm = TRUE)
 
-        if (!rlang::is_null(col_limits)) {
-          if (is.na(col_limits)[1]) col_limits[1] <- col_min
-          if (is.na(col_limits)[2]) col_limits[2] <- col_max
-        }
+      if (!rlang::is_null(col_limits)) {
+        if (is.na(col_limits)[1]) col_limits[1] <- col_min
+        if (is.na(col_limits)[2]) col_limits[2] <- col_max
+      }
 
-        if (rlang::is_null(col_limits)) col_limits <- c(col_min, col_max)
-        if (!rlang::is_null(col_include)) col_limits <- range(c(col_include, col_limits))
+      if (rlang::is_null(col_limits)) col_limits <- c(col_min, col_max)
+      if (!rlang::is_null(col_include)) col_limits <- range(c(col_include, col_limits))
 
+      if (rlang::is_null(pal)) pal <- viridis::viridis(10)
+      if (rlang::is_null(col_labels)) col_labels <- scales::label_comma()
+
+      if (col_continuous == "gradient") {
         if (rlang::is_null(col_breaks)) {
-          if (col_legend_place %in% c("b", "t")) col_breaks_n <- 3
-          else col_breaks_n <- 4
-          col_breaks <- scales::breaks_pretty(n = col_breaks_n)(col_limits)
+          if (col_legend_place %in% c("b", "t")) {
+            col_breaks <- function(x) c(x, stats::median(x))
+            draw_llim <- TRUE #should be FALSE
+            draw_ulim <- FALSE
+          }
+          else if (col_legend_place %in% c("l", "r")) {
+            col_breaks <- scales::breaks_pretty(4)
+            draw_llim <- TRUE
+            draw_ulim <- TRUE
+          }
+        } else {
+          draw_llim <- TRUE
+          draw_ulim <- TRUE
         }
 
-        if (rlang::is_null(pal)) pal <- viridis::viridis(100)
-        if (rlang::is_null(col_labels)) col_labels <- scales::label_comma()
-
         col_scale <- list(
           ggplot2::scale_colour_gradientn(
             colors = pal,
@@ -376,7 +368,12 @@ gg_tile <- function(
             breaks = col_breaks,
             limits = col_limits,
             na.value = pal_na,
-            guide = ggplot2::guide_colorbar(title.position = col_title_position)
+            guide = ggplot2::guide_colourbar(
+              title.position = col_title_position,
+              draw.ulim = draw_ulim,
+              draw.llim = draw_llim,
+              ticks.colour = "#F1F3F5",
+              reverse = col_legend_rev)
           ),
           col_scale <- ggplot2::scale_fill_gradientn(
             colors = pal,
@@ -384,66 +381,40 @@ gg_tile <- function(
             breaks = col_breaks,
             limits = col_limits,
             na.value = pal_na,
-            guide = ggplot2::guide_colorbar(title.position = col_title_position)
+            guide = ggplot2::guide_colourbar(
+              title.position = col_title_position,
+              draw.ulim = draw_ulim,
+              draw.llim = draw_llim,
+              ticks.colour = "#F1F3F5",
+              reverse = col_legend_rev)
           )
-
         )
       }
-      else { #intervals col
-        data <- data %>%
-          dplyr::mutate(dplyr::across(!!col, col_intervals))
-
-        col_levels <- levels(rlang::eval_tidy(col, data))
-        col_n <- length(col_levels)
-
-        if (rlang::is_null(pal)) pal <- pal_viridis_mix(col_n)
-        else pal <- pal[1:col_n]
-
-        if (is.numeric(rlang::eval_tidy(y, data)) |
-            lubridate::is.Date(rlang::eval_tidy(y, data))) {
-
-          if (col_legend_place %in% c("b", "t")) col_legend_rev <- FALSE
-          else col_legend_rev <- TRUE
+      else if (col_continuous == "steps") {
+        if (rlang::is_null(col_breaks)) {
+          col_breaks <- scales::breaks_pretty(n = 4)
         }
-        else if (is.factor(rlang::eval_tidy(y, data)) | is.character(rlang::eval_tidy(y, data))) {
-          if (col_legend_place %in% c("b", "t")) col_legend_rev <- TRUE
-          else col_legend_rev <- FALSE
-          pal <- rev(pal)
-        }
-        else col_legend_rev <- FALSE
-
-        if (rlang::is_null(col_breaks)) col_breaks <- ggplot2::waiver()
-        if (rlang::is_null(col_labels)) col_labels <- ggplot2::waiver()
 
         col_scale <- list(
-          ggplot2::scale_colour_manual(
-            values = pal,
-            breaks = col_levels,
-            limits = col_levels,
+          ggplot2::scale_colour_stepsn(
+            colors = pal,
             labels = col_labels,
+            breaks = col_breaks,
+            limits = col_limits,
             na.value = pal_na,
-            guide = ggplot2::guide_legend(
-              reverse = col_legend_rev,
+            guide = ggplot2::guide_coloursteps(
               title.position = col_title_position,
-              ncol = col_legend_ncol,
-              nrow = col_legend_nrow,
-              byrow = TRUE
-            )
+              reverse = col_legend_rev)
           ),
-          ggplot2::scale_fill_manual(
-            values = pal,
-            breaks = col_levels,
-            limits = col_levels,
+          col_scale <- ggplot2::scale_fill_stepsn(
+            colors = pal,
             labels = col_labels,
+            breaks = col_breaks,
+            limits = col_limits,
             na.value = pal_na,
-            # aesthetics = c("col", "fill"),
-            guide = ggplot2::guide_legend(
-              reverse = col_legend_rev,
+            guide = ggplot2::guide_coloursteps(
               title.position = col_title_position,
-              ncol = col_legend_ncol,
-              nrow = col_legend_nrow,
-              byrow = TRUE
-            )
+              reverse = col_legend_rev)
           )
         )
       }
@@ -461,28 +432,30 @@ gg_tile <- function(
       if (rlang::is_null(pal)) pal <- pal_d3_mix(col_n)
       else pal <- pal[1:col_n]
 
-      if (is.numeric(rlang::eval_tidy(y, data)) |
-          lubridate::is.Date(rlang::eval_tidy(y, data))) {
+      if (y_numeric |
+          y_date) {
 
-        if (is.character(rlang::eval_tidy(col, data)) | is.factor(rlang::eval_tidy(col, data))) {
-          col_legend_rev <- FALSE
+        if (col_character | is.factor(rlang::eval_tidy(col, data))) {
+          col_legend_rev_auto <- FALSE
         }
-        else if (col_legend_place %in% c("b", "t")) col_legend_rev <- FALSE
-        else col_legend_rev <- TRUE
+        else if (col_legend_place %in% c("b", "t")) col_legend_rev_auto <- FALSE
+        else col_legend_rev_auto <- TRUE
       }
-      else if (is.factor(rlang::eval_tidy(y, data)) | is.character(rlang::eval_tidy(y, data))) {
-        if (is.character(rlang::eval_tidy(col, data)) | is.factor(rlang::eval_tidy(col, data))) {
-          col_legend_rev <- TRUE
+      else if (y_character | y_factor) {
+        if (col_character | is.factor(rlang::eval_tidy(col, data))) {
+          col_legend_rev_auto <- TRUE
         }
-        else if (col_legend_place %in% c("b", "t")) col_legend_rev <- TRUE
-        else col_legend_rev <- FALSE
+        else if (col_legend_place %in% c("b", "t")) col_legend_rev_auto <- TRUE
+        else col_legend_rev_auto <- FALSE
         pal <- rev(pal)
       }
-      else col_legend_rev <- FALSE
+      else col_legend_rev_auto <- FALSE
 
       if (rlang::is_null(col_breaks)) col_breaks <- ggplot2::waiver()
       if (rlang::is_null(col_labels)) col_labels <- ggplot2::waiver()
 
+      if (col_legend_rev) col_legend_rev_auto <- !col_legend_rev_auto
+
       col_scale <- list(
         ggplot2::scale_colour_manual(
           values = pal,
@@ -491,7 +464,7 @@ gg_tile <- function(
           labels = col_labels,
           na.value = pal_na,
           guide = ggplot2::guide_legend(
-            reverse = col_legend_rev,
+            reverse = col_legend_rev_auto,
             title.position = col_title_position,
             ncol = col_legend_ncol,
             nrow = col_legend_nrow,
@@ -504,7 +477,7 @@ gg_tile <- function(
           labels = col_labels,
           na.value = pal_na,
           guide = ggplot2::guide_legend(
-            reverse = col_legend_rev,
+            reverse = col_legend_rev_auto,
             title.position = col_title_position,
             ncol = col_legend_ncol,
             nrow = col_legend_nrow,
@@ -515,8 +488,8 @@ gg_tile <- function(
   }
 
   ###make plot
-  if (!rlang::quo_is_null(x) & !rlang::quo_is_null(y)) {
-    if (!rlang::quo_is_null(col)) {
+  if (!x_null & !y_null) {
+    if (!col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           x = !!x,
@@ -526,7 +499,7 @@ gg_tile <- function(
           group = !!group
         ))
     }
-    else if (rlang::quo_is_null(col)) {
+    else if (col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           x = !!x,
@@ -537,8 +510,8 @@ gg_tile <- function(
         ))
     }
   }
-  else if (!rlang::quo_is_null(x) & rlang::quo_is_null(y)) {
-    if (!rlang::quo_is_null(col)) {
+  else if (!x_null & y_null) {
+    if (!col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           x = !!x,
@@ -547,7 +520,7 @@ gg_tile <- function(
           group = !!group
         ))
     }
-    else if (rlang::quo_is_null(col)) {
+    else if (col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           x = !!x,
@@ -557,8 +530,8 @@ gg_tile <- function(
         ))
     }
   }
-  else if (rlang::quo_is_null(x) & !rlang::quo_is_null(y)) {
-    if (!rlang::quo_is_null(col)) {
+  else if (x_null & !y_null) {
+    if (!col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           y = !!y,
@@ -567,7 +540,7 @@ gg_tile <- function(
           group = !!group
         ))
     }
-    else if (rlang::quo_is_null(col)) {
+    else if (col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           y = !!y,
@@ -577,8 +550,8 @@ gg_tile <- function(
         ))
     }
   }
-  else if (rlang::quo_is_null(x) & rlang::quo_is_null(y)) {
-    if (!rlang::quo_is_null(col)) {
+  else if (x_null & y_null) {
+    if (!col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           col = !!col,
@@ -586,7 +559,7 @@ gg_tile <- function(
           group = !!group
         ))
     }
-    else if (rlang::quo_is_null(col)) {
+    else if (col_null) {
       plot <- data %>%
         ggplot2::ggplot(mapping = ggplot2::aes(
           col = "",
@@ -602,29 +575,77 @@ gg_tile <- function(
       stat = stat,
       position = position,
       alpha = alpha,
-      height = height,
-      width = width,
       #linewidth = linewidth,
       ...
     )
 
-  if (!rlang::quo_is_null(facet)) {
-    if (rlang::quo_is_null(facet2)) {
+  if (rlang::is_null(facet_layout)) {
+    if (!facet_null & facet2_null) facet_layout <- "wrap"
+    else if (!facet2_null & facet_null) facet_layout <- "wrap"
+    else if (!facet_null & !facet2_null) facet_layout <- "grid"
+    else if (facet_null & facet2_null) facet_layout <- "grid"
+    else facet_layout <- "null"
+  }
+
+  if (facet_layout == "wrap") {
+    if (!facet_null & facet2_null) {
       plot <- plot +
         ggplot2::facet_wrap(
-          ggplot2::vars(!!facet),
-          scales = facet_scales, labeller = ggplot2::as_labeller(facet_labels),
+          facets = ggplot2::vars(!!facet),
+          scales = facet_scales,
+          nrow = facet_nrow,
           ncol = facet_ncol,
-          nrow = facet_nrow
+          labeller = ggplot2::as_labeller(facet_labels)
         )
     }
-    else {
+    else if (facet_null & !facet2_null) {
+      plot <- plot +
+        ggplot2::facet_wrap(
+          facets = ggplot2::vars(!!facet2),
+          scales = facet_scales,
+          nrow = facet_nrow,
+          ncol = facet_ncol,
+          labeller = ggplot2::as_labeller(facet_labels)
+        )
+    }
+    else if (!facet_null & !facet2_null) {
+      plot <- plot +
+        ggplot2::facet_wrap(
+          facets = ggplot2::vars(!!facet, !!facet2),
+          scales = facet_scales,
+          nrow = facet_nrow,
+          ncol = facet_ncol,
+          labeller = ggplot2::as_labeller(facet_labels)
+        )
+    }
+  }
+  else if (facet_layout == "grid") {
+    if (!facet_null & !facet2_null) {
       plot <- plot +
         ggplot2::facet_grid(
           rows = ggplot2::vars(!!facet2),
           cols = ggplot2::vars(!!facet),
-          labeller = ggplot2::as_labeller(facet_labels),
-          scales = facet_scales
+          scales = facet_scales,
+          space = facet_space,
+          labeller = ggplot2::as_labeller(facet_labels)
+        )
+    }
+    else if (!facet_null & facet2_null) {
+      plot <- plot +
+        ggplot2::facet_grid(
+          cols = ggplot2::vars(!!facet),
+          scales = facet_scales,
+          space = facet_space,
+          labeller = ggplot2::as_labeller(facet_labels)
+        )
+    }
+    else if (facet_null & !facet2_null) {
+      plot <- plot +
+        ggplot2::facet_grid(
+          rows = ggplot2::vars(!!facet2),
+          scales = facet_scales,
+          space = facet_space,
+          labeller = ggplot2::as_labeller(facet_labels)
         )
     }
   }
@@ -642,7 +663,7 @@ gg_tile <- function(
   layer_data <- ggplot2::layer_data(plot)
 
   ###Make x scale based on layer_data
-  if (is.character(rlang::eval_tidy(x, data)) | is.factor(rlang::eval_tidy(x, data))) {
+  if (x_character | x_factor) {
     if (rlang::is_null(x_expand)) x_expand <- ggplot2::waiver()
     if (rlang::is_null(x_labels)) x_labels <- ggplot2::waiver()
 
@@ -656,7 +677,7 @@ gg_tile <- function(
         tidyr::pivot_longer(cols = tidyselect::everything()) %>%
         dplyr::pull(.data$value)
 
-      if (lubridate::is.Date(rlang::eval_tidy(x, data))) {
+      if (x_date) {
         x_vctr <- as.Date(x_vctr, origin = "1970-01-01")
       }
 
@@ -668,25 +689,45 @@ gg_tile <- function(
         if (!rlang::is_null(x_include)) x_limits <- range(c(x_limits, x_include))
 
         if (rlang::is_null(x_breaks)) {
-          x_breaks_n <- ifelse(rlang::quo_is_null(facet), 5, 3)
+          x_breaks_n <- ifelse(facet_null, 5, 3)
           if (x_trans != c("identity")) x_breaks <- scales::breaks_log(n = x_breaks_n, base = 10)(x_limits)
           else x_breaks <- scales::breaks_pretty(n = x_breaks_n)(x_limits)
 
-          if (xy_numeric_date) x_limits <- NULL
+          if ((x_numeric | x_date) & (y_date | y_numeric | y_null)) x_limits <- NULL
           else {
             if (x_trans != "identity") x_limits <- NULL
+            else if (class(position)[1] == "PositionFill") x_limits <- NULL
+            else if (class(position)[1] == "character") {
+              if (position[1] == "fill") x_limits <- NULL
+              else x_limits <- c(min(x_breaks), max(x_breaks))
+            }
             else x_limits <- c(min(x_breaks), max(x_breaks))
           }
         }
         else if (!rlang::is_null(x_breaks)) {
-          if (xy_numeric_date) x_limits <- NULL
+          if ((x_numeric | x_date) & (y_date | y_numeric | y_null)) x_limits <- NULL
           else {
             if (is.vector(x_breaks)) {
               if (x_trans != "identity") x_limits <- NULL
+              else if (class(position)[1] == "PositionFill") x_limits <- NULL
+              else if (class(position)[1] == "character") {
+                if (position[1] == "fill") x_limits <- NULL
+                else x_limits <- c(min(x_breaks), max(x_breaks))
+              }
               else x_limits <- c(min(x_breaks), max(x_breaks))
             }
             else {
               if (x_trans != "identity") x_limits <- NULL
+              else if (class(position)[1] == "PositionFill") x_limits <- NULL
+              else if (class(position)[1] == "character") {
+                if (position[1] == "fill") x_limits <- NULL
+                else {
+                  x_limits <- list(x_limits) %>%
+                    purrr::map(.f = x_breaks) %>%
+                    unlist() %>%
+                    range()
+                }
+              }
               else {
                 x_limits <- list(x_limits) %>%
                   purrr::map(.f = x_breaks) %>%
@@ -703,7 +744,7 @@ gg_tile <- function(
         if (!rlang::is_null(x_include)) x_limits <- range(c(x_limits, x_include))
 
         if (rlang::is_null(x_breaks)) {
-          x_breaks_n <- ifelse(rlang::quo_is_null(facet), 5, 4)
+          x_breaks_n <- ifelse(facet_null, 5, 4)
           if (x_trans != "identity") x_breaks <- scales::breaks_log(n = x_breaks_n, base = 10)(x_limits)
           else x_breaks <- scales::breaks_pretty(n = x_breaks_n)(x_limits)
         }
@@ -715,7 +756,7 @@ gg_tile <- function(
 
     if (rlang::is_null(x_expand)) {
       if (facet_scales %in% c("fixed", "free_y")) {
-        if (xy_numeric_date) {
+        if ((x_numeric | x_date) & (y_date | y_numeric | y_null)) {
           x_expand <- c(0.05, 0.05)
         }
         else x_expand <- c(0, 0)
@@ -724,28 +765,30 @@ gg_tile <- function(
     }
 
     if (rlang::is_null(x_labels)) {
-      if (is.numeric(rlang::eval_tidy(x, data)) | rlang::quo_is_null(x)) x_labels <- scales::label_comma()
-      else if (lubridate::is.Date(rlang::eval_tidy(x, data))) x_labels <- scales::label_date_short()
+      if (x_numeric | x_null) x_labels <- scales::label_comma()
+      else if (x_date) x_labels <- scales::label_date_short()
       else x_labels <- ggplot2::waiver()
     }
 
-    if (is.numeric(rlang::eval_tidy(x, data)) | rlang::quo_is_null(x)) {
+    if (x_numeric | x_null) {
       x_scale <- ggplot2::scale_x_continuous(
         breaks = x_breaks,
         limits = x_limits,
         expand = x_expand,
         labels = x_labels,
-        oob = x_oob,
+        oob = scales::oob_keep,
+        sec.axis = x_sec_axis,
         trans = x_trans
       )
     }
-    else if (lubridate::is.Date(rlang::eval_tidy(x, data))) {
+    else if (x_date) {
       x_scale <- ggplot2::scale_x_date(
         breaks = x_breaks,
         limits = x_limits,
         expand = x_expand,
         labels = x_labels,
-        oob = x_oob
+        oob = scales::oob_keep,
+        sec.axis = x_sec_axis
       )
     }
   }
@@ -754,7 +797,7 @@ gg_tile <- function(
     x_scale
 
   ###Make y scale based on layer_data
-  if (is.character(rlang::eval_tidy(y, data)) | is.factor(rlang::eval_tidy(y, data))) {
+  if (y_character | y_factor) {
     if (rlang::is_null(y_expand)) y_expand <- ggplot2::waiver()
     if (rlang::is_null(y_labels)) y_labels <- ggplot2::waiver()
 
@@ -763,11 +806,11 @@ gg_tile <- function(
   else {
     if (facet_scales %in% c("fixed", "free_x")) {
       y_vctr <- layer_data %>%
-        dplyr::select(tidyselect::matches(stringr::regex("^y$|^ymin$|^ymax$|^yend$|^ymay_final$"))) %>%
+        dplyr::select(tidyselect::matches(stringr::regex("^y$|^ymin$|^ymax$|^yend$|^ymax_final$"))) %>%
         tidyr::pivot_longer(cols = tidyselect::everything()) %>%
         dplyr::pull(.data$value)
 
-      if (lubridate::is.Date(rlang::eval_tidy(y, data))) {
+      if (y_date) {
         y_vctr <- as.Date(y_vctr, origin = "1970-01-01")
       }
 
@@ -779,20 +822,40 @@ gg_tile <- function(
         if (!rlang::is_null(y_include)) y_limits <- range(c(y_limits, y_include))
 
         if (rlang::is_null(y_breaks)) {
-          y_breaks_n <- ifelse(rlang::quo_is_null(facet), 5, 3)
+          y_breaks_n <- ifelse(facet_null, 5, 3)
           if (y_trans != c("identity")) y_breaks <- scales::breaks_log(n = y_breaks_n, base = 10)(y_limits)
           else y_breaks <- scales::breaks_pretty(n = y_breaks_n)(y_limits)
 
           if (y_trans != "identity") y_limits <- NULL
+          else if (class(position)[1] == "PositionFill") y_limits <- NULL
+          else if (class(position)[1] == "character") {
+            if (position[1] == "fill") y_limits <- NULL
+            else y_limits <- c(min(y_breaks), max(y_breaks))
+          }
           else y_limits <- c(min(y_breaks), max(y_breaks))
         }
         else if (!rlang::is_null(y_breaks)) {
           if (is.vector(y_breaks)) {
             if (y_trans != "identity") y_limits <- NULL
+            else if (class(position)[1] == "PositionFill") y_limits <- NULL
+            else if (class(position)[1] == "character") {
+              if (position[1] == "fill") y_limits <- NULL
+              else y_limits <- c(min(y_breaks), max(y_breaks))
+            }
             else y_limits <- c(min(y_breaks), max(y_breaks))
           }
           else {
             if (y_trans != "identity") y_limits <- NULL
+            else if (class(position)[1] == "PositionFill") y_limits <- NULL
+            else if (class(position)[1] == "character") {
+              if (position[1] == "fill") y_limits <- NULL
+              else {
+                y_limits <- list(y_limits) %>%
+                  purrr::map(.f = y_breaks) %>%
+                  unlist() %>%
+                  range()
+              }
+            }
             else {
               y_limits <- list(y_limits) %>%
                 purrr::map(.f = y_breaks) %>%
@@ -808,7 +871,7 @@ gg_tile <- function(
         if (!rlang::is_null(y_include)) y_limits <- range(c(y_limits, y_include))
 
         if (rlang::is_null(y_breaks)) {
-          y_breaks_n <- ifelse(rlang::quo_is_null(facet), 5, 4)
+          y_breaks_n <- ifelse(facet_null, 5, 4)
           if (y_trans != "identity") y_breaks <- scales::breaks_log(n = y_breaks_n, base = 10)(y_limits)
           else y_breaks <- scales::breaks_pretty(n = y_breaks_n)(y_limits)
         }
@@ -829,28 +892,30 @@ gg_tile <- function(
     }
 
     if (rlang::is_null(y_labels)) {
-      if (is.numeric(rlang::eval_tidy(y, data)) | rlang::quo_is_null(x)) y_labels <- scales::label_comma()
-      else if (lubridate::is.Date(rlang::eval_tidy(y, data))) y_labels <- scales::label_date_short()
+      if (y_numeric | y_null) y_labels <- scales::label_comma()
+      else if (y_date) y_labels <- scales::label_date_short()
       else y_labels <- ggplot2::waiver()
     }
 
-    if (is.numeric(rlang::eval_tidy(y, data)) | rlang::quo_is_null(y)) {
+    if (y_numeric | y_null) {
       y_scale <- ggplot2::scale_y_continuous(
         breaks = y_breaks,
         limits = y_limits,
         expand = y_expand,
         labels = y_labels,
-        oob = y_oob,
+        oob = scales::oob_keep,
+        sec.axis = y_sec_axis,
         trans = y_trans
       )
     }
-    else if (lubridate::is.Date(rlang::eval_tidy(y, data))) {
+    else if (y_date) {
       y_scale <- ggplot2::scale_y_date(
         breaks = y_breaks,
         limits = y_limits,
         expand = y_expand,
         labels = y_labels,
-        oob = y_oob
+        oob = scales::oob_keep,
+        sec.axis = y_sec_axis,
       )
     }
   }
@@ -878,7 +943,7 @@ gg_tile <- function(
     plot <- plot +
       ggplot2::theme(legend.direction = "horizontal")
 
-    if (is.numeric(rlang::eval_tidy(col, data))) {
+    if (col_numeric) {
       plot <- plot +
         ggplot2::theme(legend.key.width = grid::unit(0.66, "cm")) +
         ggplot2::theme(legend.text.align = 0.5)
@@ -894,7 +959,7 @@ gg_tile <- function(
     }
   }
 
-  else if (col_legend_place == "n" | rlang::quo_is_null(col)) {
+  else if (col_legend_place == "n" | col_null) {
     plot <- plot +
       ggplot2::theme(legend.position = "none")
   }
@@ -908,6 +973,6 @@ gg_tile <- function(
       ggplot2::theme(legend.position = "right")
   }
 
-  #return beautiful plot
+  plot <- plot + ggplot2::theme(legend.justification = "left") #return beautiful plot
   return(plot)
 }
